// Arduino + Scratch ヘルパー
// サンハヤトAS-E406用
// このヘルパーに対応したArduinoファームウェアと
// Scratch 拡張機能が必要です
// このjsはNodejsとexpress,serialportが必要です
// node app.js COM3 と指定することでArduinoのポートが指定できます
// by Takao Takahashi
// https://tsq.gapj.net

//--以下の値は使用する環境に応じて変更します
// Arduino接続パラメータ
// Arduinoが使用するポート
var arduino_port = 'COM4';
const arduino_speed = 115200;
//Arduino UNO R3では115200bpsで問題なく動作しています

// Scratch機能拡張用ポート番号
const exten_listen = 8004;


//各種変数
var pinVals = [];
var report_string = "";


// expressを使用する
const express = require('express')
      , routes = require('./routes')
      , path = require('path');

const app = express();

//シリアルポート指定
if(process.argv[2] != null) arduino_port = process.argv[2];
console.log('Arduino at ' + arduino_port);


//シリアルポートを使用する
const SerialPort = require('serialport');
//シリアルポートからの読み込みは行単位
const Readline = SerialPort.parsers.Readline;
//シリアルポートの各種設定
const port = new SerialPort(arduino_port,{
		baudRate: arduino_speed});
//パーサとしてReadlineを使う
const parser = new Readline();
port.pipe(parser);

//シリアルポートからの受信処理
//シリアルポートから来たデータを変換してScratch Exten.へ送出
parser.on('data', function(data){
  //デバッグ用
  //console.log('--' + data);
  //Arduinoからのデータはコロン区切り
  pinVals = data.split(':');
  //Scratchへのリポート文字列を初期化
  report_string = '';
  //各要素について処理
  pinVals.forEach(function(pindata){
    var tmp_str  = pindata.split(' ');
    //リポート文字列は"digitalValue/D1 1"の形式にして出力
    //頭がDならデジタルピン
    if(tmp_str[0].charAt(0) == 'D'){
      report_string += 'digitalValue/' + pindata + '\n';
      //D8とD9はスイッチ
      if(tmp_str[0].charAt(1) == '8'){
        report_string += 'switch2 ' + tmp_str[1] + '\n';
      }
      if(tmp_str[0].charAt(1) == '9'){
        report_string += 'switch1 ' + tmp_str[1] + '\n';
      }
    }
    //頭がAならアナログピン
    if(tmp_str[0].charAt(0) == 'A'){
      report_string += 'analogValue/' + pindata + '\n';
      //A0は明るさなので明るさデータもセット
      if(tmp_str[0].charAt(1) == '0'){
        report_string += 'light ' + tmp_str[1] + '\n';
      }
    }
    //頭がTなら温度
    if(tmp_str[0].charAt(0) == 'T'){
      var tmp_data = pindata.split(' ');
      report_string += 'tempreture ' + tmp_data[1] + '\n';
    }
  });
});

//このヘルパーのHTTPでの待機
//Scratch HTTP拡張のため8001で待ち受ける
const server = app.listen(exten_listen, function(){
    console.log('exten listen at.' + server.address().port);
});


//Scratchからポーリングされた場合に呼ばれる処理
app.get('/poll', (req, res) => {
  //ポーリングに対して各ピンの情報を送出
  res.send(report_string);
});


//Scratchからのリセット要求
app.get('/rest_all', (req, res) => {
  //今のところ何もしない
  console.log('Reset Recieved');
  res.send('OK');
});


//Scratch各ブロックに対応した処理

//ピン値設定(デジタル)
app.get('/pinval/:pinNum/:pinVal', (req, res) => {
  //console.log("Pin " + req.params.pinNum);
  //console.log("To " + req.params.pinVal);
  var tmp_val = '0';
  if(req.params.pinVal.toUpperCase() == 'HIGH') tmp_val = '1';
  if(req.params.pinVal.toUpperCase() == 'LOW') tmp_val = '0';
  var tmp_str = req.params.pinNum + ' ' + tmp_val + '\n';
  port.write(tmp_str);
  res.send('OK');
});

//LED制御
app.get('/ledcont/:ledNum/:pinVal', (req, res) => {
  var tmp_val = '0';
  var tmp_led = '10';
  if(req.params.pinVal.toUpperCase() == 'HIGH') tmp_val = '1';
  if(req.params.pinVal.toUpperCase() == 'LOW') tmp_val = '0';
  if(req.params.ledNum.toUpperCase() == 'LED1') tmp_led = 'D11';
  if(req.params.ledNum.toUpperCase() == 'LED2') tmp_led = 'D10';
  var tmp_str = tmp_led + ' ' + tmp_val + '\n';
  port.write(tmp_str);
  res.send('OK');
});

//PWM出力
app.get('/pinpwm/:pinNum/:pinVal', (req, res) => {
  var tmp_val = req.params.pinVal;
  var tmp_str = req.params.pinNum.replace('D','A');
  if(Number(tmp_val) > 255) tmp_val = '255';
  //console.log("Pin " + tmp_str);
  //console.log("To " + tmp_val);
  if(tmp_val > 255) tmp_val = '255';
  tmp_str = tmp_str + ' ' + tmp_val + '\n';
  port.write(tmp_str);
  res.send('OK');
});

//ピンモード設定
app.get('/pinmode/:pinNum/:pinMode', (req, res) => {
  //console.log("Pin " + req.params.pinNum);
  //console.log("To " + req.params.pinMode);
  var tmp_val = '0';
  var tmp_str = req.params.pinNum.replace('D','M');
  if(req.params.pinMode.toUpperCase() == 'OUTPUT') tmp_val = '1';
  if(req.params.pinMode.toUpperCase() == 'INPUT') tmp_val = '0';
  tmp_str = tmp_str + ' ' + tmp_val + '\n';
  port.write(tmp_str);;
  res.send('OK');
});

//気温測定開始
app.get('/gettemppress', (req, res) => {
  port.write('T' + '\n');
  res.send('OK');
});
